@@ -1,5 +1,6 @@
 """Main module."""
 
+from concurrent.futures import ThreadPoolExecutor
 import requests
 import logging
 import os
@@ -377,7 +378,7 @@ class client():
     def cmpd_search(self, corpNameList="", corpNameFrom="", corpNameTo="",
                     aliasContSelect="contains", alias="", dateFrom="",
                     dateTo="", searchType="substructure", percentSimilarity=90,
-                    chemist="anyone", maxResults=100, molStructure=""
+                    chemist="anyone", maxResults=100, molStructure="", projectCodes=None
                     ):
         if isinstance(corpNameList, list):
             corpNameList = ",".join(corpNameList)
@@ -401,11 +402,14 @@ class client():
                              % VALID_STRUCTURE_SEARCH_TYPES)
         return self.cmpd_structure_search_request(search_request)
 
-    def get_all_lots(self):
+    def get_all_lots(self, project_codes=None):
         """Get all lots
 
         Get all lots the currently logged in user is allowed to access
 
+        Args:
+            project_codes (list): A list of project codes to filter by
+
         Returns: Returns an array of dict objects
             id (id): the lot corp name
             lotCorpName (str): the lot corp name
@@ -417,7 +421,51 @@ class client():
         resp = self.session.get("{}/cmpdReg/parentLot/getAllAuthorizedLots"
                                 .format(self.url))
         resp.raise_for_status()
-        return resp.json()
+
+        lots = resp.json()
+
+        if project_codes is not None:
+            lots = [lot for lot in lots if lot["project"] in project_codes]
+        return lots
+
+
+    def get_all_meta_lots(self, project_codes=None):
+        """Get all meta lots
+
+        Get all meta lots the currently logged in user is allowed to access
+
+        Args:
+            project_codes (list): A list of project codes to filter by
+
+        Returns: Returns an array of dict objects respresenting meta lots
+        """
+        
+        # Get all lots
+        lots = self.get_all_lots(project_codes=project_codes)
+
+        # Get meta lots
+        meta_lots = self.get_meta_lots_by_lot_corp_names([lot["lotCorpName"] for lot in lots])
+
+        return meta_lots
+        
+    def get_meta_lots_by_lot_corp_names(self, lot_corp_names, max_workers=10):
+        """Get meta lots by lot corp names
+
+        Get meta lots by lot corp names
+
+        Args:
+            lot_corp_names (list): A list of lot corp names
+
+        Returns: Returns an array of dict objects respresenting meta lots
+        """
+        meta_lots = []
+
+        # Call get_meta_lot for each lot corp name
+        # Run max_workers requests in parallel
+        with ThreadPoolExecutor(max_workers=max_workers) as executor:
+            for meta_lot in executor.map(self.get_meta_lot, lot_corp_names):
+                meta_lots.append(meta_lot)
+        return meta_lots
 
     def cmpd_search_request(self, search_request):
         search_request["loggedInUser"] = self.username
@@ -447,6 +495,8 @@ class client():
             search_request["chemist"] = "anyone"
         if("maxResults" not in search_request):
             search_request["maxResults"] = 100
+        if("projectCodes" in search_request and search_request["projectCodes"] is None):
+            del search_request["projectCodes"]
 
         resp = self.session.post("{}/cmpdreg/search/cmpds".
                                  format(self.url),
@@ -1006,9 +1056,28 @@ class client():
         resp.raise_for_status()
         return resp.json()
 
-    def experiment_search(self, query):
+    def experiment_search(self, query, project_codes=None):
+        """Search for experiments by search term
+
+        Get an array of experiments given an experiment search term string and optional project code(s) filter
+
+        Args:
+            query (str): An experiment search term
+            project_codes (str list): A list of project codes to filter by
+
+en array of protocols
+        """
+
+        params = {}
+        if project_codes is not None:
+            # Convert to a comma separated string and url encode
+            params['projectCodes'] = ','.join(project_codes)
+
         resp = self.session.get("{}/api/experiments/genericSearch/{}/"
-                                .format(self.url, query))
+                                .format(self.url, query),
+                                params=params)
+
+        resp.raise_for_status()
         return resp.json()
 
     def get_cmpdreg_bulk_load_files(self):
@@ -1056,6 +1125,40 @@ class client():
             return None
         return resp.json()
 
+    def get_lot_corp_names_by_bulk_load_file(self, id):
+        """Get an array of lot corp names from a bulk load file
+
+        Args:
+            id (int): A bulk load file id
+
+        Returns: An array of lot corp names
+
+        """
+        resp = self.session.get("{}/api/cmpdRegBulkLoader/getLotsByBulkLoadFileID/{}".
+                                format(self.url,
+                                       id))
+        resp.raise_for_status()
+        if resp.text == '"Error"':
+            return None
+        return resp.json()
+
+    def get_sdf_by_bulk_load_file(self, id):
+        """Get an SDF file from a bulk load file id
+
+        Args:
+            id (int): A bulk load file id
+
+        Returns: (str) representaiton of the SDF file
+
+        """
+        resp = self.session.get("{}/api/cmpdRegBulkLoader/getSDFFromBulkLoadFileId/{}".
+                                format(self.url,
+                                       id))
+        resp.raise_for_status()
+        if resp.text == '"Error"':
+            return None
+        return resp.text
+
     def purge_cmpdreg_bulk_load_file(self, id):
         """Purge a cmpdreg bulk load file
 
